import { Component, h, State, Element, Listen, Prop, Watch } from '@stencil/core';
import { getSize } from '../../utils/size';
import { getLangData, useLanguage } from '../../lib/lang';
import { defaultIconPack, getIconPack } from '../../lib/icons';
import { defaultConfig } from '../../lib/default-ui-config';
import { Render } from '../../lib/render';
import { provideDyteDesignSystem } from '../../index';
import { getUserPreferences } from '../../utils/user-prefs';
export class DyteMeeting {
  constructor() {
    this.roomJoinedListener = () => {
      this.setStates({ meeting: 'joined' });
    };
    this.roomLeftListener = ({ state }) => {
      if (this.states.roomLeftState === 'disconnected') {
        this.setStates({ meeting: 'ended' });
        return;
      }
      this.setStates({ meeting: 'ended', roomLeftState: state });
    };
    /** Where to show setup screen or not */
    this.showSetupScreen = true;
    /** Language */
    this.language = 'en';
    /** UI Config */
    this.config = defaultConfig;
    this.states = {
      meeting: 'idle',
      prefs: {
        mirrorVideo: true,
        muteNotificationSounds: false,
      },
    };
    this.iconPack = defaultIconPack;
    this.t = useLanguage();
  }
  connectedCallback() {
    this.resizeObserver = new ResizeObserver(() => this.handleResize());
    this.resizeObserver.observe(this.host);
    if (typeof this.config.designTokens !== 'undefined') {
      provideDyteDesignSystem(this.host, this.config.designTokens);
    }
    this.meetingChanged(this.meeting);
    this.iconPackUrlChanged(this.iconPackUrl);
    this.languageChanged(this.language);
    this.initializePreferences();
  }
  disconnectedCallback() {
    var _a, _b;
    this.resizeObserver.disconnect();
    this.roomJoinedListener &&
      ((_a = this.meeting) === null || _a === void 0 ? void 0 : _a.self.removeListener('roomJoined', this.roomJoinedListener));
    this.roomLeftListener && ((_b = this.meeting) === null || _b === void 0 ? void 0 : _b.self.removeListener('roomLeft', this.roomLeftListener));
  }
  meetingChanged(meeting) {
    if (meeting != null) {
      if (this.showSetupScreen) {
        this.states = Object.assign(Object.assign({}, this.states), { meeting: 'setup' });
      }
      meeting.self.addListener('roomJoined', this.roomJoinedListener);
      meeting.self.addListener('roomLeft', this.roomLeftListener);
      if (!this.showSetupScreen) {
        // join directly to the meeting
        meeting.joinRoom();
      }
    }
  }
  async iconPackUrlChanged(url) {
    this.iconPack = await getIconPack(url);
  }
  async languageChanged(lang) {
    this.t = useLanguage(await getLangData(lang));
  }
  listenState(e) {
    e.stopPropagation();
    this.setStates(e.detail);
  }
  listenViewImage(e) {
    this.image = e.detail;
  }
  initializePreferences() {
    const prefs = getUserPreferences();
    this.setStates({ prefs });
  }
  handleResize() {
    this.size = getSize(this.host.clientWidth);
  }
  setStates(states) {
    this.states = Object.assign(Object.assign({}, this.states), states);
  }
  render() {
    const defaults = {
      meeting: this.meeting,
      size: this.size,
      states: this.states,
      config: this.config,
      iconPack: this.iconPack,
      t: this.t,
    };
    return (h(Render, { element: "dyte-meeting", defaults: defaults, asHost: true }, this.image && (h("dyte-image-viewer", Object.assign({}, defaults, { image: this.image, onClose: () => (this.image = null) })))));
  }
  static get is() { return "dyte-meeting"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["dyte-meeting.css"]
  }; }
  static get styleUrls() { return {
    "$": ["dyte-meeting.css"]
  }; }
  static get properties() { return {
    "meeting": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "Meeting",
        "resolved": "DyteClient",
        "references": {
          "Meeting": {
            "location": "import",
            "path": "../../types/dyte-client"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Meeting object"
      }
    },
    "showSetupScreen": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Where to show setup screen or not"
      },
      "attribute": "show-setup-screen",
      "reflect": true,
      "defaultValue": "true"
    },
    "language": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "Language",
        "resolved": "\"en\" | \"hi\"",
        "references": {
          "Language": {
            "location": "import",
            "path": "../../lib/lang"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Language"
      },
      "attribute": "language",
      "reflect": true,
      "defaultValue": "'en'"
    },
    "config": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "UIConfig",
        "resolved": "UIConfig",
        "references": {
          "UIConfig": {
            "location": "import",
            "path": "../../types/ui-config"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "UI Config"
      },
      "defaultValue": "defaultConfig"
    },
    "iconPackUrl": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Icon Pack URL"
      },
      "attribute": "icon-pack-url",
      "reflect": true
    },
    "size": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "Size",
        "resolved": "\"lg\" | \"md\" | \"sm\" | \"xl\"",
        "references": {
          "Size": {
            "location": "import",
            "path": "../../types/props"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Size"
      },
      "attribute": "size",
      "reflect": true
    }
  }; }
  static get states() { return {
    "states": {},
    "iconPack": {},
    "t": {},
    "image": {}
  }; }
  static get elementRef() { return "host"; }
  static get watchers() { return [{
      "propName": "meeting",
      "methodName": "meetingChanged"
    }, {
      "propName": "iconPackUrl",
      "methodName": "iconPackUrlChanged"
    }, {
      "propName": "language",
      "methodName": "languageChanged"
    }]; }
  static get listeners() { return [{
      "name": "dyteStateUpdate",
      "method": "listenState",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "dyteViewImage",
      "method": "listenViewImage",
      "target": undefined,
      "capture": false,
      "passive": false
    }]; }
}
