/**
 * Converts Hex color codes to RGB strings for use in the design system
 * @param {string} hex The hex code of the color
 * @returns Space separated RGB values
 */
const hexToRGB = (hex) => {
  const r = parseInt(hex.slice(1, 3), 16);
  const g = parseInt(hex.slice(3, 5), 16);
  const b = parseInt(hex.slice(5, 7), 16);

  return `${r} ${g} ${b}`;
};

/**
 * Function which adds support for adding any opacity value to colors from CSS Variables
 * @param {string} varName The CSS variable name of the color
 * @param {string} color The color code in Hex
 * @param {number} opacityValue The opacity value - [0,100]
 * @returns Parsed string to be used in Tailwind CSS config
 */
const getColorWithOpacity = (varName, color, opacityValue) => {
  if (opacityValue !== undefined) {
    return `rgba(var(${varName}, ${hexToRGB(color)}) / ${opacityValue})`;
  }
  return `rgb(var(${varName}, ${hexToRGB(color)}))`;
};

module.exports = ({ colors }) => ({
  inherit: colors.inherit,
  current: colors.current,
  transparent: colors.transparent,
  brand: {
    DEFAULT: ({ opacityValue }) =>
      getColorWithOpacity('--dyte-colors-brand-500', '#2160FD', opacityValue),
    300: ({ opacityValue }) =>
      getColorWithOpacity('--dyte-colors-brand-300', '#497CFD', opacityValue),
    400: ({ opacityValue }) =>
      getColorWithOpacity('--dyte-colors-brand-400', '#356EFD', opacityValue),
    500: ({ opacityValue }) =>
      getColorWithOpacity('--dyte-colors-brand-500', '#2160FD', opacityValue),
    600: ({ opacityValue }) =>
      getColorWithOpacity('--dyte-colors-brand-600', '#0D51FD', opacityValue),
    700: ({ opacityValue }) =>
      getColorWithOpacity('--dyte-colors-brand-700', '#0246FD', opacityValue),
  },
  background: {
    DEFAULT: ({ opacityValue }) =>
      getColorWithOpacity('--dyte-colors-background-1000', '#080808', opacityValue),
    1000: ({ opacityValue }) =>
      getColorWithOpacity('--dyte-colors-background-1000', '#080808', opacityValue),
    900: ({ opacityValue }) =>
      getColorWithOpacity('--dyte-colors-background-900', '#1A1A1A', opacityValue),
    800: ({ opacityValue }) =>
      getColorWithOpacity('--dyte-colors-background-800', '#1E1E1E', opacityValue),
    700: ({ opacityValue }) =>
      getColorWithOpacity('--dyte-colors-background-700', '#2C2C2C', opacityValue),
    600: ({ opacityValue }) =>
      getColorWithOpacity('--dyte-colors-background-600', '#3C3C3C', opacityValue),
  },
  text: {
    1000: 'rgb(var(--dyte-colors-text-1000, 255 255 255))',
    900: 'rgb(var(--dyte-colors-text-900, 255 255 255 / 0.88))',
    800: 'rgb(var(--dyte-colors-text-800, 255 255 255 / 0.76))',
    700: 'rgb(var(--dyte-colors-text-700, 255 255 255 / 0.64))',
    600: 'rgb(var(--dyte-colors-text-600, 255 255 255 / 0.52))',
  },
  'video-bg': ({ opacityValue }) =>
    getColorWithOpacity('--dyte-colors-video-bg', '#181818', opacityValue),
  danger: ({ opacityValue }) =>
    getColorWithOpacity('--dyte-colors-danger', '#FF2D2D', opacityValue),
  success: ({ opacityValue }) =>
    getColorWithOpacity('--dyte-colors-success', '#83D017', opacityValue),
  warning: ({ opacityValue }) =>
    getColorWithOpacity('--dyte-colors-warning', '#FFCD07', opacityValue),
});
